/**************************************************/
/*                                                */
/*                  RSCTOC  V1.0                  */
/*                                                */
/*  Copyright (c) 1988, by Robert Joseph Silliker */
/*               All Rights Reserved              */
/*     Permission is granted for unrestricted     */
/*              non-commercial use                */
/*                                                */
/**************************************************/

/**************************************************/
/*                                                */
/*          GEMMED by Les Kneeling 1989           */
/*                                                */
/*  Apart from making it easier for morons like   */
/*  me to use the program, this listing can help  */
/*  to give an idea of how to use the output from */
/*                     RSCTOC                     */
/*                                                */
/**************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <osbind.h>
#include <gemdefs.h>
#include <obdefs.h>
#include <define.h>

#include "rsctoc.h"

extern OBJECT *rs_trindex[];

static char *copyright[] = "RSCTOC V1.0\nCopyright (c) 1988, by\
 Robert Joseph Silliker\nAll Rights Reserved\n\
 Permission is granted for unrestricted non-commercial use\n";

long _stksize = 16000;

RSHDR *rschead;                /* Pointer to the resource file header. */
char *out_path;                /* Path to output file. */
char *rsc_path;                /* Path to input file.  */
FILE *fp = stdout;             /* Default output file. */
char *prefix= "";              /* Prefix to the names of arrays. */

#define MAXTREES 512           /* Maximum number of object trees per file. */

OBJECT *objects;               /* Pointer to the base of the objects. */
OBJECT *trindex[MAXTREES];     /* One pointer per object tree. */
OBJECT *dialog_box;            /* Dialog used by RSCTOC */
OBJECT *help_box;              /* Help dialog box */
char *progname = "rsctoc.ttp"; /* Program name (no argv[0]) */

main()
{
int x, y, w, h;

appl_init();                    /* initialise roms                  */
fix_objects();                  /* use routine to sort out resources*/
dialog_box = rs_trindex[0];     /* get address of object tree       */
help_box = rs_trindex[1];       /* and the help tree                */
init_dialog();                  /* clear the text fields            */
graf_mouse( ARROW, 0l );
draw_dialog( x, y, w, h);       /* display the dialog               */
handle_form( x, y, w, h);       /* use form_do to interact with it  */
undraw_dialog( x, y, w, h);     /* get rid of the dialog box        */
appl_exit();                    /* clean up and go                  */
}

init_dialog()
{
int drv;
drv = Dgetdrv() + 'A';
sprintf( ((TEDINFO *)dialog_box[SOURCE].ob_spec)->te_ptext, "%c:\\", drv);
sprintf( ((TEDINFO *)dialog_box[DEST].ob_spec)->te_ptext, "%c:\\", drv );
sprintf( ((TEDINFO *)dialog_box[PREFIX].ob_spec)->te_ptext, "" );
}

draw_dialog( x, y, w, h)
int x, y, w, h;
{
form_center( dialog_box, &x, &y, &w, &h );
form_dial( FMD_START, 0, 0, 0, 0, x, y, w, h );
objc_draw( dialog_box, 0, MAX_DEPTH, x, y, w, h );
}

handle_form( x, y, w, h)
int x, y, w, h;
{
int event;
int hx, hy, hw, hh;
do  {
    event = form_do( dialog_box, SOURCE );

    switch( event )
        {
        case SOURCE:
             select( SOURCE );
             objc_draw( dialog_box, 0, MAX_DEPTH, 0, 0, 640, 400 );
             break;
        case DEST:
             select( DEST );
             objc_draw( dialog_box, 0, MAX_DEPTH, 0, 0, 640, 400 );
             break;
        case HELP:
             form_center( help_box, &hx, &hy, &hw, &hh );
             objc_draw( help_box, 0, MAX_DEPTH, 0, 0, 640, 400 );
             form_do( help_box, 0 );
             help_box[HELPEXIT].ob_state &= ~SELECTED;
             dialog_box[HELP].ob_state &= ~SELECTED;
             objc_draw( dialog_box, 0, MAX_DEPTH, 0, 0, 640, 400 );
             break;
        case CONVERT:
             if( check_fields() )
                {
                graf_mouse( HOURGLASS, 0l );
                rsc_path = ((TEDINFO *)dialog_box[SOURCE].ob_spec)->te_ptext;
                prefix = ((TEDINFO *)dialog_box[PREFIX].ob_spec)->te_ptext;
                out_path = ((TEDINFO *)dialog_box[DEST].ob_spec)->te_ptext;
                if(!init_prog()) 
                    {
                    put_header();
                    make_tedinfo();
                    make_icon();
                    make_bitblk();
                    make_object();
                    make_index();
                    make_fix();
                    }
                clean_up();
                graf_mouse( ARROW, 0l );
                }
             else
                name_error();
             objc_change( dialog_box, CONVERT, 0, 0, 0, 640, 400, NORMAL, 1 );
             break;
        }
    }while( event != FINISHED );
}

select( which )
int which;
{
char path[160], name[15];
char temp[160];
int button, count;
static int last;

path[0] = Dgetdrv() + 'A';
path[1] = ':';
path[2] = 0;
if( which == SOURCE )
    strcat( path, "\\*.RSC" );  /* set wildcards for source file     */
else
    strcat( path, "\\*.C" );    /* set wildcards for dest file       */

if( last != which )             /* been here before?                 */
    {
    last = which;               /* if this is the first time through */
    strcpy( name, "" );         /* clear the text in "name"          */
    }
fsel_input( path, name, &button );  /* choose file               */
if( button == 1 )
    {
    count = strlen( path );         /* get rid of the wildcards  */
    while( path[count] != '\\' )
        count--;
    path[count+1] = 0;
    sprintf( ((TEDINFO *)dialog_box[which].ob_spec)->te_ptext, "%s%s",
             path, name );          /* put path and name into    */
    }                               /* the dialog box            */
}

check_fields()  /* must have source and dest filenames */
{
return( ( strlen( ((TEDINFO *)dialog_box[SOURCE].ob_spec)->te_ptext ) > 1 )
        & ( strlen( ((TEDINFO *)dialog_box[DEST].ob_spec)->te_ptext ) > 1 ));
}

name_error()
{
form_alert( 1, "[1][ Must specify source| and destination file| names][OK]" );
}

undraw_dialog( x, y, w, h)
int x, y, w, h;
{
form_dial( FMD_FINISH, 0, 0, 0, 0, x, y, w, h );
}

/*-------------------------*/
/* Initialize the program. */
/*-------------------------*/

int init_prog()
{
  int i, errstat = FALSE;
  char temp[160];

if(!myrsrc_load(rsc_path))
    {
    sprintf( temp, "[1][Error getting resource file.][ OK ]" );
    form_alert( 1, temp );
    errstat = TRUE;
    }
else if(out_path != "") {
    if((fp = fopen(out_path, "w")) == 0)
      {
      sprintf( temp, "[1][Can't open output file.][ OK ]" );
      form_alert( 1, temp );
      errstat = TRUE;
      }
    }
else if(rschead->rsh_nobs <= 0)
    {
    sprintf( temp, "[1][No object trees to dump.][ OK ]" );
    form_alert( 1, temp );
    errstat = TRUE;
    }
return(errstat);
}


/*--------------------------------------------*/
/* Clean up open files                        */
/*--------------------------------------------*/

clean_up()
{

  if((fp != stdout) && (fp != 0))
    fclose(fp);
}


/*------------------------------*/
/* Extract tedinfo information. */
/*------------------------------*/

make_tedinfo()
{
  int i, first_one;
  register OBJECT *g;
  TEDINFO *t;

  fprintf(fp, "TEDINFO %srs_tedinfo[] = {\n", prefix);

  for(i=0, first_one=1, g=objects; i < rschead->rsh_nobs; i++, g++) {
    if(g->ob_type == G_FTEXT || g->ob_type == G_FBOXTEXT ||
       g->ob_type == G_TEXT  || g->ob_type == G_BOXTEXT) {

      if(first_one) {
       first_one = 0;
      }
      else
        fprintf(fp, ",\n");

      t = (TEDINFO *)(g->ob_spec);

      if(strlen(t->te_ptext))
        fprintf(fp, "  {\"%s\", ", t->te_ptext);
      else
        fprintf(fp, "  {%sp_to_nul, ", prefix);

      if(strlen(t->te_ptmplt))
        fprintf(fp, "\"%s\", ", t->te_ptmplt);
      else
        fprintf(fp, "%sp_to_nul, ", prefix);

      if(strlen(t->te_pvalid))
        fprintf(fp, "\"%s\",\n  ", t->te_pvalid);
      else
        fprintf(fp, "%sp_to_nul,\n   ", prefix);

      fprintf(fp, "%3d, ", t->te_font);
      fprintf(fp, "%3d, ", t->te_junk1);
      fprintf(fp, "%3d, ", t->te_just);
      fprintf(fp, "%3d, ", t->te_color);
      fprintf(fp, "%3d, ", t->te_junk2);
      fprintf(fp, "%3d, ", t->te_thickness);
      fprintf(fp, "%3d, ", t->te_txtlen);
      fprintf(fp, "%3d}", t->te_tmplen);
    }
  }

  if(!first_one)
    fprintf(fp, "\n};\n\n");
  else
    fprintf(fp, "  0L\n};\n\n");
}


/*---------------------------*/
/* Extract icon information. */
/*---------------------------*/

make_icon()
{
  int i, j, k, numb;
  int first_one, *p;
  register OBJECT *g;
  ICONBLK *icon;

  for(i=0, numb=0, g=objects; i < rschead->rsh_nobs; i++, g++) {
    /*----------------------*/
    /* First do the images. */
    /*----------------------*/

    if(g->ob_type == G_ICON) {
      icon = (ICONBLK *)(g->ob_spec);

      fprintf(fp, "int %simask%03d[] = {", prefix, numb);
      k = icon->ib_hicon * (icon->ib_wicon/16);

      for(j=0, p=icon->ib_pmask; k > 0; j++, p++, k--) {
        if(!(j % 8))
          fprintf(fp, "\n  ");
        fprintf(fp, "0x%04x", *p);

       if(k != 1)
          fprintf(fp, ",");
      }
      fprintf(fp, "\n};\n\n");

      fprintf(fp, "int %sidata%03d[] = {", prefix, numb);
      k = icon->ib_hicon * (icon->ib_wicon/16);

      for(j=0, p=icon->ib_pdata; k > 0; j++, p++, k--) {
        if(!(j % 8))
         fprintf(fp, "\n   ");
        fprintf(fp, "0x%04x", *p);

        if(k != 1)
         fprintf(fp, ",");
      }

      fprintf(fp, "\n};\n\n");
      numb++;
    }
  }
 
  fprintf(fp, "ICONBLK %srs_iconblk[] = {\n", prefix);

  for(i=0, numb=0, g=objects, first_one=1; i < rschead->rsh_nobs; i++, g++) {
    /*-----------------------------*/
    /* Now do the icon structures. */
    /*-----------------------------*/

    if(g->ob_type == G_ICON) {

      if(first_one)
        first_one = 0;
      else
        fprintf(fp, ",\n");

      icon = (ICONBLK *)(g->ob_spec);

      fprintf(fp, "  {%simask%03d, %sidata%03d, ", prefix, numb, prefix, numb);

      if(strlen(icon->ib_ptext))
        fprintf(fp, "\"%s\",\n", icon->ib_ptext);
      else
        fprintf(fp, "%sp_to_nul, \n", prefix);

      fprintf(fp, "   0x%04x, 0x%04x, 0x%04x, 0x%04x, 0x%04x, 0x%04x,\n",
                 icon->ib_char, icon->ib_xchar, icon->ib_ychar,
                  icon->ib_xicon, icon->ib_yicon, icon->ib_wicon);

      fprintf(fp, "   0x%04x, 0x%04x, 0x%04x, 0x%04x, 0x%04x}",
                 icon->ib_hicon, icon->ib_xtext, icon->ib_ytext,
                 icon->ib_wtext, icon->ib_htext);
      numb++;
    }
  }

  if(!first_one)
    fprintf(fp, "\n};\n\n");
  else
    fprintf(fp, "  0L\n};\n\n");
}



/*-----------------------------*/
/* Extract bitblk information. */
/*-----------------------------*/

make_bitblk()
{
  int i, j, k, numb;
  int first_one, *p;
  register OBJECT *g;
  BITBLK *bitblk;

  for(i=0, numb=0, g=objects; i < rschead->rsh_nobs; i++, g++) {
    /*----------------------*/
    /* First do the images. */
    /*----------------------*/

    if(g->ob_type == G_IMAGE) {
      bitblk = (BITBLK *)(g->ob_spec);

      fprintf(fp, "int %sbblock%03d[] = {", prefix, numb);
      k = bitblk->bi_hl * (bitblk->bi_wb/2);

      for(j=0, p=bitblk->bi_pdata; k > 0; j++, p++, k--) {
        if(!(j % 8))
          fprintf(fp, "\n  ");
        fprintf(fp, "0x%04x", *p);

       if(k != 1)
          fprintf(fp, ",");
      }
      fprintf(fp, "\n};\n\n");

      numb++;
    }
  }

  fprintf(fp, "BITBLK %srs_bitblk[] = {\n", prefix);
 
  for(i=0, numb=0, g=objects, first_one=1; i < rschead->rsh_nobs; i++, g++) {
    /*-------------------------------*/
    /* Now do the bitblk structures. */
    /*-------------------------------*/

    if(g->ob_type == G_IMAGE) {

      if(first_one)
        first_one = 0;
      else
        fprintf(fp, ",\n");

      bitblk = (BITBLK *)(g->ob_spec);

      fprintf(fp, "  {%sbblock%03d, ", prefix, numb);

      fprintf(fp, " 0x%04x, 0x%04x, 0x%04x, 0x%04x, 0x%04x}",
                           bitblk->bi_wb, bitblk->bi_hl, bitblk->bi_x,
                            bitblk->bi_y, bitblk->bi_color);

      numb++;
    }
  }

  if(!first_one)
    fprintf(fp, "\n};\n\n");
  else
    fprintf(fp, "  0L\n};\n\n");
}


/*-----------------------------------------------*/
/* Extract object information and change into C. */
/*-----------------------------------------------*/

make_object()
{
  int i, ic, bb, ted;
  int first_one;
  register OBJECT *g;

  for(i=0,ic=0,bb=0,ted=0,g=objects,first_one=1; i<rschead->rsh_nobs;i++,g++) {
    if(first_one) {
      fprintf(fp, "OBJECT %srs_object[] = {\n", prefix);
      first_one = 0;
    }
    else
      fprintf(fp, ",\n");

    print_index_comment(g);

    fprintf(fp, "  {%4d,%4d,%4d,0x%04x,0x%04x,0x%04x, ",
                                g->ob_next, g->ob_head, g->ob_tail,
                                g->ob_type, g->ob_flags, g->ob_state);

    if(g->ob_type == G_STRING || g->ob_type == G_TITLE
                              || g->ob_type == G_BUTTON) {

      if(strlen(g->ob_spec))
        fprintf(fp, "\"%s\",", g->ob_spec);
      else
        fprintf(fp, "%sp_to_nul,", prefix);
    }
    else if(g->ob_type == G_FTEXT || g->ob_type == G_FBOXTEXT
                                  || g->ob_type == G_TEXT
                                  || g->ob_type == G_BOXTEXT) {
      fprintf(fp, "\&%srs_tedinfo[%d],", prefix, ted);
      ted++;
    }
    else if(g->ob_type == G_ICON) {
      fprintf(fp, "\&%srs_iconblk[%d],", prefix, ic);
      ic++;
    }
    else if(g->ob_type == G_IMAGE) {
      fprintf(fp, "\&%srs_bitblk[%d],", prefix, bb);
      bb++;
    }
    else
      fprintf(fp, "0x%08lxL,", g->ob_spec);

    fprintf(fp, "%4d,%4d,%4d,%4d}",
               g->ob_x, g->ob_y, g->ob_width, g->ob_height);
  }

  if(!first_one)
    fprintf(fp, "\n};\n\n");
}


/*-----------------------------------------------*/
/* Build the array of tree pointers and generate */
/* the structure size macros.                    */
/*-----------------------------------------------*/

make_index()
{
  int i, index, first_one;

  for(i=0, first_one=1; i < rschead->rsh_ntree; i++) {
    if(first_one) {
      fprintf(fp, "OBJECT *%srs_trindex[] = {\n", prefix);
      first_one = 0;
    }
    else
      fprintf(fp, ",\n");

    index = (trindex[i] - objects);

    fprintf(fp, "  &%srs_object[%d]", prefix, index);
  }
  if(!first_one)
    fprintf(fp, "\n};\n\n");

  fprintf(fp, "#define %sNUM_TI %d\n", prefix, rschead->rsh_nted);
  fprintf(fp, "#define %sNUM_IB %d\n", prefix, rschead->rsh_nib);
  fprintf(fp, "#define %sNUM_BB %d\n", prefix, rschead->rsh_nbb);
  fprintf(fp, "#define %sNUM_OBS %d\n", prefix, rschead->rsh_nobs);
  fprintf(fp, "#define %sNUM_TREE %d\n\n\n", prefix, rschead->rsh_ntree);
}


/*------------------------------------------------------*/
/* Output the include #ifdef for the object definition  */
/* header file.  Output a single NUL character pointer  */
/* into the C source.  This will be used as the pointer */
/* to an empty character string wherever strings have   */
/* zero length.                                         */
/*------------------------------------------------------*/

put_header()
{
  fprintf(fp, "#ifndef OBJECT\n");
  fprintf(fp, "#include <obdefs.h>\n");
  fprintf(fp, "#endif\n\n");
  fprintf(fp, "static char %sp_to_nul[1] = \"\";\n\n", prefix);
}


/*-------------------------------------------*/
/* Output code that will fix up the objects. */
/*-------------------------------------------*/

make_fix()
{
  fprintf(fp, "%sfix_objects()\n{\n", prefix);
  fprintf(fp, "  register int i;\n\n");
  fprintf(fp, "  for(i=0; i<%sNUM_OBS; i++)\n", prefix);
  fprintf(fp, "    rsrc_obfix(%srs_object, i);\n", prefix);
  fprintf(fp, "}\n\n");
}


/*--------------------------------*/
/* Open and read in the rsc file. */
/* Then call the routine to fix   */
/* the object pointers.           */
/*--------------------------------*/

char *pbase;                   /* Pointer to base of file in memory */
#define BLKSIZE 8192           /* Number of bytes per read call */
                               /* Don't make this larger than 32767 */

myrsrc_load(path)
char *path;
{
  long fsize, lseek();
  int rsc, size;
  char *p;
  char temp[160];

  if((rsc = open(path, O_RDONLY | O_BINARY)) < 0) {
    sprintf(temp, "[1][%s: Can't open '%s'.][ OK ]", progname, path);
    form_alert( 1, temp );
    return(0);
  }

  if((fsize = lseek(rsc, 0L, 2)) < 0)
    return(0);

  lseek(rsc, 0L, 0);

  if(fsize < sizeof(RSHDR)) {
    sprintf(temp, "[1][%s: Resource file is too small.][ OK ]", progname);
    form_alert( 1, temp );
    close(rsc);
    return(0);
  }

  p = pbase = (char *)(Malloc(fsize));
  if(pbase == 0l) {
    sprintf(temp, "[1][%s: Not enough memory.][ OK ]", progname);
    form_alert( 1, temp );
    close(rsc);
    return(0);
  }

  do {
    size = (fsize >= BLKSIZE) ? BLKSIZE : (int)(fsize);
    if(read(rsc, p, size) != size) {
      sprintf(temp, "[1][%s: Read error on '$s'.][ OK ]", progname, path);
      form_alert( 1, temp );
      Mfree(pbase);
      close(rsc);
      return(0);
    }
    p += size;
    fsize -= (long)(size);
  } while(fsize > 0L);

  close(rsc);

  fix_pointers();

  return(1);
}


/*------------------------------------------*/
/* Routine to fix up the resource pointers. */
/*------------------------------------------*/

fix_pointers()
{
  int i;
  long *t;
  /*----------------------------------------------------*/
  /* Initialize the pointer to the resource file header */
  /* and a pointer to the base of the object array.     */
  /*----------------------------------------------------*/

  rschead = (RSHDR *)(pbase);
  objects = (OBJECT *)(pbase + rschead->rsh_object);

  t = (long *)(pbase + rschead->rsh_trindex);  /* Get pointer to trindex */

  for(i=0; i<rschead->rsh_ntree; i++)
    trindex[i] = (OBJECT *)(pbase + *(t+i));   /* Build tree pointer */

  /*---------------------------------------------------*/
  /* Fix all the objects.  No need to go into the maze */
  /* of head, tail, next because all objects are in a  */
  /* nice array of size 'rsh_nobs'.                    */
  /*---------------------------------------------------*/

  for(i=0; i<rschead->rsh_nobs; i++)
    fix_one_object(objects + i);
}


/*-------------------------------------*/
/* Fix up the pointers for one object. */
/*-------------------------------------*/

fix_one_object(g)
register OBJECT *g;
{
  TEDINFO *t;
  ICONBLK *icon;
  BITBLK *bitblk;

  switch(g->ob_type) {
    case(G_STRING):
    case(G_TITLE):
    case(G_BUTTON):
      g->ob_spec = (char *)(pbase + (long)(g->ob_spec));
      break;
    case(G_FTEXT):
    case(G_FBOXTEXT):
    case(G_TEXT):
    case(G_BOXTEXT):
      t = (TEDINFO *)(pbase + (long)(g->ob_spec));
      g->ob_spec = (char *)(t);
      t->te_ptext = (char *)(pbase + (long)(t->te_ptext));
      t->te_ptmplt = (char *)(pbase + (long)(t->te_ptmplt));
      t->te_pvalid = (char *)(pbase + (long)(t->te_pvalid));
      break;
    case(G_ICON):
      icon = (ICONBLK *)(pbase + (long)(g->ob_spec));
      g->ob_spec = (char *)(icon);
      icon->ib_pmask = (int *)(pbase + (long)(icon->ib_pmask));
      icon->ib_pdata = (int *)(pbase + (long)(icon->ib_pdata));
      icon->ib_ptext = (char *)(pbase + (long)(icon->ib_ptext));
      break;
    case(G_IMAGE):
      bitblk = (BITBLK *)(pbase + (long)(g->ob_spec));
      g->ob_spec = (char *)(bitblk);
      bitblk->bi_pdata = (int *)(pbase + (long)(bitblk->bi_pdata));
      break;
  }
}


/*---------------------------------*/
/* Print the rs_trindex[] comment. */
/*---------------------------------*/

print_index_comment(g)
OBJECT *g;
{
  register int i;

  for(i=0; i<rschead->rsh_ntree; i++) {
    if(trindex[i] == g)
      fprintf(fp, "  /* Tree #%d*/\n", i);
  }
}

/* END OF FILE */

